"""
This module provides task scheduling classes for the management of OmniTracker
SRR (NHRR) processing for Department UMH.
    SRR: Sustainability Risk Rating
    NHRR: Nachhaltigkeits Risiko Rating
"""
import os
# import os.PathLike
import shutil

from ka_uts_uts.utils.pac import Pac

from typing import Any

TyPathLike = os.PathLike
TyArr = list[Any]
TyBool = bool
TyDic = dict[Any, Any]
# TyAoA = list[TyArr]
# TyAoD = list[TyDic]
# TyDoAoA = dict[Any, TyAoA]
# TyDoAoD = dict[Any, TyAoD]

# TyPath = str
# TyPath = str | TyPathLike[str] | bytes | TyPathLike[bytes]

TnAny = None | Any
# TnAoD = None | TyAoD
TnDic = None | TyDic
# TnPath = None | TyPath


class Setup:
    """
    Setup function class
    """
    @classmethod
    def setup(cls, kwargs: TyDic) -> None:
        _package: str = kwargs.get('package', '')
        _src_dir_app: Any = kwargs.get('src_dir_app', '')
        _src_dir_dat: Any = kwargs.get('src_dir_dat', '')
        _tgt_dir_app: Any = kwargs.get('tgt_dir_app', '')
        _tgt_dir_dat: Any = kwargs.get('tgt_dir_dat', '')
        _pac_src_dir_app: Any = Pac.sh_path_by_package(_package, _src_dir_app)
        _pac_src_dir_dat: Any = Pac.sh_path_by_package(_package, _src_dir_dat)
        print(f" setup _pac_src_dir_app = {_pac_src_dir_app}")
        print(f" setup _pac_src_dir_dat = {_pac_src_dir_dat}")
        cls.copytree(_pac_src_dir_app, _tgt_dir_app)
        cls.copytree(_pac_src_dir_dat, _tgt_dir_dat)

    @staticmethod
    def copytree(src: Any, tgt: Any) -> None:
        if not src:
            return
        if not os.path.exists(tgt):
            os.makedirs(tgt)
        shutil.copytree(src, tgt, dirs_exist_ok=True)



# coding=utf-8
from typing import Any

import os

from ka_uts_uts.utils.pac import Pac

TyArr = list[Any]
TyDic = dict[Any, Any]
TyPath = str


class PacMod:
    """ Package Module Management
    """
    @staticmethod
    def sh_d_pacmod(cls) -> TyDic:
        """ Show Pacmod Dictionary
        """
        a_pacmod: TyArr = cls.__module__.split(".")
        return {'package': a_pacmod[0], 'module': a_pacmod[1]}

    @staticmethod
    def sh_path_module_yaml(d_pacmod: TyDic) -> Any:
        """ show directory
        """
        package = d_pacmod['package']
        module = d_pacmod['module']
        path = f"{module}/data/{module}.yml"
        return Pac.sh_path_by_package(package, path)

    @staticmethod
    def sh_path_keys(d_pacmod: TyDic) -> Any:
        """ show directory
        """
        package = d_pacmod['package']
        path = 'data/keys.yml'
        return Pac.sh_path_by_package(package, path)

    # @staticmethod
    # def sh_path_log_cfg(com) -> TyPath:
    #     """ show directory
    #     """
    #     package = com.d_app_pacmod['package']
    #     path = resources.files(package).joinpath(f"data/log.{com.log_type}.yml")
    #     if path.is_file():
    #         return path
    #     package = com.d_com_pacmod['package']
    #     path = resources.files(package).joinpath(f"data/log.{com.log_type}.yml")
    #     if path.is_file():
    #         return path
    #     raise ModuleNotFoundError

    # @staticmethod
    # def sh_path_cfg(d_pacmod: TyDic) -> Any:
    #     """ show directory
    #     """
    #     package = com.d_app_pacmod['package']
    #     path = 'dat/cfg.yml'
    #     return Pac.sh_path(package, path)

    # @staticmethod
    # def sh_path_type(d_pacmod: TyDic, type_: str) -> str:
    #     """ show Data File Path
    #     """
    #     # def sh_pacmod_type(d_pacmod: TyDic, type_: str) -> str:
    #     package = d_pacmod['package']
    #     module = d_pacmod['module']
    #     return f"/data/{package}/{module}/{type_}"

    # @classmethod
    # def sh_file_path(
    #         cls, d_pacmod: TyDic, type_: str, suffix: str,
    #         pid: Any, ts: Any, **kwargs) -> str:
    #     """ show type specific path
    #     """
    #     filename_ = kwargs.get('filename', type_)
    #     sw_run_pid_ts = kwargs.get('sw_run_pid_ts', True)
    #     if sw_run_pid_ts is None:
    #         sw_run_pid_ts = True
    #
    #   _dir: str = cls.sh_pacmod_type(d_pacmod, type_)
    #   if sw_run_pid_ts:
    #       file_path = os_path.join(
    #           _dir, f"{filename_}_{pid}_{ts}.{suffix}")
    #   else:
    #       file_path = os_path.join(_dir, f"{filename_}.{suffix}")
    #   return file_path

    @staticmethod
    def sh_dir_type(com, type_: str) -> TyPath:
        """Show run_dir
        """
        dir_dat: str = com.dir_dat
        tenant: str = com.tenant
        package: str = com.d_app_pacmod['package']
        module: str = com.d_app_pacmod['module']
        if not tenant:
            return f"{dir_dat}/{tenant}/{package}/{module}/{type_}"
        else:
            return f"{dir_dat}/{package}/{module}/{type_}"

    @classmethod
    def sh_path_pattern(
            cls, com, filename, type_: str, suffix: str) -> TyPath:
        """ show type specific path
        """
        _dir: str = cls.sh_dir_type(com, type_)
        return os.path.join(_dir, f"{filename}*.{suffix}")



# coding=utf-8
import yaml

from typing import Any
TyAny = Any
TyArr = list[Any]
TyDic = dict[Any, Any]

TnAny = None | Any


class Yaml_:
    """ Manage Object to Yaml file affilitation
    """
    @staticmethod
    def read(path: str, log) -> TnAny:
        try:
            with open(path) as fd:
                # The Loader parameter handles the conversion from YAML
                # scalar values to Python object format
                obj = yaml.load(fd, Loader=yaml.SafeLoader)
                return obj
        except FileNotFoundError:
            log.error(f"No such file or directory: path='{path}'")
            raise
        except IOError:
            # if Com.Log is not None:
            #     fnc_error = Com.Log.error
            #     fnc_error(exc, exc_info=True)
            raise
        return None



# coding=utf-8
from typing import Any

import os
import importlib.resources as resources

TyArr = list[Any]
TyDic = dict[Any, Any]
TyPackage = str
TyPackages = list[str]
TyPath = str


class Pac:

    @staticmethod
    def sh_path_by_package(package: TyPackage, path: TyPath) -> Any:
        """ show directory
        """
        _path = str(resources.files(package).joinpath(path))
        # if _path.is_file():
        if _path is not None and _path != '':
            if os.path.exists(_path):
                return _path
        return ''

    @classmethod
    def sh_path_by_packages(cls, packages: TyPackages, path: TyPath) -> Any:
        """ show directory
        """
        for _package in packages:
            _path = cls.sh_path_by_package(_package, path)
            if _path:
                return _path
        return ''



# coding=utf-8
# from collections.abc import Callable
from typing import Any

import os
import yaml
import jinja2
# import logging
# import logging.config
from logging import Logger

TyAny = Any
TyDic = dict[Any, Any]
TyLogger = Logger
TyStr = str

TnDic = None | TyDic
TnStr = None | TyStr


class Jinja2_:

    """ Manage Object to Json file affilitation
    """
    @staticmethod
    def read_template(path: TyStr) -> Any:
        directory, file = os.path.split(path)
        env = jinja2.Environment(loader=jinja2.FileSystemLoader(directory))
        return env.get_template(file)

    @classmethod
    def read(cls, path: TyStr, log: TyLogger, **kwargs) -> Any:
        try:
            # read jinja template from file
            template = cls.read_template(path)
            # render template as yaml string
            template_rendered = template.render(kwargs)
            # load yaml string into object
            return yaml.safe_load(template_rendered)
        except IOError as exc:
            log.critical(exc, exc_info=True)
            # log.error(f"No such file or directory: path='{path'}")
            raise
